#ifndef _CAPTURE_H
#define _CAPTURE_H

#include <GL/gl.h>
#include <vector>

class Capture {
public:
int screenWidth, screenHeight;
int captureframecount;

bool captureEnabled;
uint32_t* capturePixels;
FILE * capturePipe;


Capture( int w, int h ) : screenWidth(w), screenHeight(h)
{
	captureframecount = 0;
	captureEnabled = false;
	capturePixels = 0;
	capturePipe = 0;
	//    framecount = 0;
}


// call this before glutswapbuffers
void update()
{
	// frame dumping
	if (captureEnabled)
	{
		glReadPixels( 0, 0, screenWidth, screenHeight, GL_RGBA, GL_UNSIGNED_BYTE, capturePixels );
		captureframecount++;
		write_capturepipe( capturePixels );
	}
}

void write_capturepipe( uint32_t* pixels )
{
	fprintf(capturePipe, "P6\n%d %d\n255\n", screenWidth, screenHeight );

	uint8_t * pix8 = (uint8_t*)pixels;

	unsigned char scanline[8192];

	for(int y=screenHeight-1; y>=0; y--)
	{
		for( int x=0; x<screenWidth; x++)
		{
			scanline[x*3+0] = pix8[y*screenWidth*4+x*4+0];
			scanline[x*3+1] = pix8[y*screenWidth*4+x*4+1];
			scanline[x*3+2] = pix8[y*screenWidth*4+x*4+2];
		}
		fwrite( scanline, 3*screenWidth, 1, capturePipe );
	}
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	screenWidth = w;
	screenHeight = h;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1, 1, -1, 1, -2.0, 2.0);
	glMatrixMode(GL_MODELVIEW);
	if (captureEnabled)
	{
		disableCapture();
		enableCapture();
	}
}

void disableCapture()
{
	captureEnabled = false;
	if (capturePixels)
		free(capturePixels);

	if (capturePipe)
	{
		pclose( capturePipe );
		capturePipe = NULL;
	}
}

void enableCapture()
{
	captureEnabled = true;
	capturePixels = (uint32_t*) malloc( sizeof(uint32_t) * screenHeight * screenWidth );
	//    capturePipe = popen("ffmpeg -f image2pipe -vcodec ppm -an -r 25.0 -b 4000 -i pipe: -vcodec mpeg1video -y capture.mpg", "w" );
	//    capturePipe = popen("ffmpeg -f image2pipe -vcodec ppm -an -r 25.0 -b 16000 -i pipe: -vcodec mpeg4 -me zero -y capture.avi", "w" );
	//      capturePipe = popen("ffmpeg -i DISCODEL.wav  -f image2pipe -vcodec ppm -r 25.0 -b 16000 -i pipe: -vcodec mpeg4  -target dvd -me zero -y capture.mpg", "w" );
	capturePipe = popen("ffmpeg -an  -f image2pipe -vcodec ppm -r 25.0 -b 16000 -i pipe: -vcodec mpeg4  -target dvd -me zero -y capture.mpv", "w" );
	if (!capturePipe)
	{
		printf("failed to start capture. ffmpeg failed to execute!\n");
		perror("popen ffmpeg:");
		disableCapture();
	}
}

};

#endif
