#include "sound.h"


#include "fmod.hpp"
#include "fmod_errors.h"

void ERRCHECK(FMOD_RESULT result)
{
	if (result != FMOD_OK)
	{
		printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
		exit(-1);
	}
}

void SoundSystem::init()
{
	int key;
	unsigned int version;

	/*
	   Create a System object and initialize.
	 */
	result = FMOD::System_Create(&system);
	ERRCHECK(result);

	result = system->getVersion(&version);
	ERRCHECK(result);

	if (version < FMOD_VERSION)
	{
		printf("Error!  You are using an old version of FMOD %08x.  This program requires %08x\n", version, FMOD_VERSION);
		exit(2);
	}

	result = system->init(1, FMOD_INIT_NORMAL, 0);
	ERRCHECK(result);
}

void SoundSystem::load( std::string fn )
{
	result = system->createSound(fn.c_str(), FMOD_SOFTWARE | FMOD_2D | FMOD_CREATESTREAM, 0, &sound);
	ERRCHECK(result);



}

void SoundSystem::play()
{
	result = system->playSound(FMOD_CHANNEL_FREE, sound, false, &channel);
	ERRCHECK(result);
}

// return false if done playing
bool SoundSystem::update()
{
	bool playing=false;
	system->update();
	if (channel)
	{
		channel->isPlaying(&playing);

		result = channel->getSpectrum(spectrum, SPECTRUMSIZE, 0, FMOD_DSP_FFT_WINDOW_TRIANGLE);
		ERRCHECK(result);

	}
	return !playing;
}
