#ifndef _TEXTOVERLAY_H
#define _TEXTOVERLAY_H


#include <vector>
#include "texture.h"
#include <Vec2d.h>
#include <Vec3d.h>
#include "util.h"
#include "fontrenderer.h"

using namespace CGLA;
using namespace std;

class TextOverlay
{
private:
double starttime, endtime;
Vec3d color;
string text;
Vec2d position;
int halign, valign;
double scale;
public:
TextOverlay( double starttime, double endtime, Vec2d pos, double scale, string text, Vec3d color=Vec3d(1,1,1), int halign=0, int valign=0 ) :
	starttime(starttime),
	endtime(endtime),
	position( pos ),
	scale(scale),
	text( text ),
	color( color ),
	halign( halign ),
	valign( valign )
{

};

void draw( FontRenderer * renderer, double time )
{
	if (time<starttime || time  > endtime )
		return;
	double fadetime = 1.0;
	double opacity;
	opacity = max( min( min( time-starttime, endtime-time ), fadetime ), 0.0 );
	glPushMatrix();
	glTranslatef(position[0],position[1],0);
	glScalef( scale,scale, 1.0 );
	//      glColor4f( color[0], color[1], color[2], opacity );
	Vec3d outlinecolor ( 0.16,0.29,0.34 );
	renderer->draw( text, color, outlinecolor, opacity, halign, valign );
	glPopMatrix();
}
};


class TextOverlayManager
{
private:
FontRenderer frenderer;

vector<TextOverlay> overlays;

public:

TextOverlayManager() :
	frenderer( "fonts/Blomster.ttf" )
{
};

void add( TextOverlay t )
{
	overlays.push_back( t );
}

void draw( float time )
{
	for( int i=0; i<overlays.size(); i++)
	{
		overlays[i].draw( &frenderer, time);
	}
};
};

#endif
