#ifndef _TEXTURE_H
#define _TEXTURE_H

#include <GL/gl.h>
#include <GL/glu.h>
#include <string>

#include <Magick++.h>

class Texture {
public:
Texture( std::string fn )
{
	id = 0xffffffff; // means not uploaded yet.

	// load ppm file.
	width=0;
	height=0;
	max=0;

	loadmagich( fn );
}

~Texture()
{
	free(image);
}

void loadmagich( std::string fn )
{
	Magick::Image mimage( fn );
	width = mimage.size().width();
	height = mimage.size().height();
	image = (unsigned char*)malloc( width*height*4 );

	mimage.write( 0, 0, width, height, "RGBA", Magick::CharPixel, (void*)image );
}

void loadppm( std::string fn )
{
	FILE * fp = fopen( fn.c_str(), "rb");
	int res;
	char header[100];
	fgets( header, sizeof(header), fp ); // should be P6
	fgets( header, sizeof(header), fp );
	sscanf( header, "%d %d", &width, &height );
	fgets( header, sizeof(header), fp );
	sscanf( header, "%d", &max );
	image = (unsigned char*)malloc( width* height * 4 );
	res=fread( image, 1, width*height*3, fp );
	for(int x=0; x<width*height; x++)
	{
		image[x*4+0] = image[x*3+0];
		image[x*4+1] = image[x*3+1];
		image[x*4+2] = image[x*3+2];
	}
	if (res != width*height*3 )
	{
		perror("fread texture failed:");
		exit(1);
	}

	fclose(fp);
}

void upload()
{
	glGenTextures(1, &id);
	glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,  GL_MODULATE );

	glBindTexture( GL_TEXTURE_2D, id );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	//      glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width,height, 0,
	//                         GL_RGB, GL_UNSIGNED_BYTE, image );
	gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, width,height,
	                   GL_RGBA, GL_UNSIGNED_BYTE, image );
	glBindTexture( GL_TEXTURE_2D, 0 );
}

void bind()
{
	if (id == 0xffffffff)
		upload();

	glEnable(GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, id );
}

void unbind()
{
	glBindTexture( GL_TEXTURE_2D, 0 );
	glDisable( GL_TEXTURE_2D );
}
private:
GLuint id;
int width, height, max;
unsigned char * image;
};

#endif
