#include <GL/gl.h>
#include <GL/glut.h>

#include <stdio.h>
#include <stdlib.h>

#include <cmath>

#include <sys/time.h>

#include <Vec2d.h>
#include <Mat2x2d.h>

#include "plant.h"
#include "texture.h"

#include "sound.h"

#include "fontrenderer.h"
#include "backdrop.h"
#include "capture.h"
#include "texture.h"
#include "textoverlay.h"

using namespace CGLA;

Vec2d cameraFocus;
double cameraZoomFactor = 1.0;
double targetZoomFactor = 1.0;

int capturefake = 0; // has to be set from startup!

#ifdef WIN32
#include <mmsystem.h>

double ftime()
{
	static int first = 1;
	static double starttime  = 0;
	if (first)
	{
		first = 0;
		starttime = ftime();
	}

	DWORD msec = timeGetTime();


	double d = msec / 1000.0;
	return d - starttime;
}
#else

double ftime()
{
	static int first = 1;
	static double starttime  = 0;
	if (first)
	{
		first = 0;
		starttime = ftime();
	}
	struct timeval tm;
	gettimeofday( &tm, NULL );

	double d = tm.tv_sec*1.0;

	d += tm.tv_usec / 1000000.0;
	return d -starttime;
}
#endif

Capture* capture;



int g_screenWidth, g_screenHeight;


void reshape( int w, int h )
{
	printf("reshape to %dx%d\n", w,h );
	g_screenWidth = w;
	g_screenHeight = h;
	glViewport( 0,0, w,h );
	capture->reshape( w,h );
}

static void clear_and_init() {
	glClearColor(0.4,0.4,0.9,0.0);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho( -13.3, 13.3, -10, 10, -1, 1 );
	//  gluPerspective( 25, 1.0 ,0.1, 100.0 );
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	//  gluLookAt( 0.5,-3.0,2,  0,0,0,  0, 0,1 );
}


//void buildIsoSurface( float time );

// SimpelPlant proot;
SlingerPlant proot;
SoundSystem ssystem;
Backdrop backdrop;
FontRenderer* frend;
TextOverlayManager* toverlays;
Vec2d smoothCamera;

void drawBackdrop( float time, float delta )
{
	backdrop.update( time, delta );
	backdrop.draw();
}

class Planet
{
public:
//  Texture gras;
Texture planeet,stel;

Planet() :
	planeet("planeet.png"),
	stel("stel.png")
	// : gras("bkglayer.png")
{
	for(int x=0; x<3; x++)
		plants.push_back( new SimpelPlant() );
};

~Planet()
{
	for (int i=0; i<plants.size(); i++)
		delete( plants[i] );
};

void update( float time, float delta )
{
	now = time;
	for (int i=0; i<plants.size(); i++)
		plants[i]->update(time,delta);
}

void draw()
{
	int i;
	glPushMatrix();
	glRotatef( now * 3.0,0,0,1 );
	glScalef(0.4,0.4,1.0);
	for (i=0; i<plants.size(); i++)
	{
		glRotatef( 360/plants.size(),0,0,1 );
		double cheat = 0.3;
//         if (i==1)
//           cheat = 0.3;
//         if (i==0)
//           cheat = 0.2;
		plants[i]->draw( Vec2d( 0,2-cheat ), Mat2x2d( 1,0,0,1) );
	}

	planeet.bind();
	glBegin(GL_QUADS);
	glColor3f(0.2,0.5,0.3);
	glTexCoord2f( 0, 0 );
	glVertex2f( -2, -2 );
	glTexCoord2f( 1, 0 );
	glVertex2f( 2, -2 );
	glTexCoord2f( 1, 1 );
	glVertex2f( 2, 2 );
	glTexCoord2f( 0, 1 );
	glVertex2f( -2, 2 );
	glEnd();
	planeet.unbind();

	glPushMatrix();
	double stelsize = 0.3;
	glRotatef( 90/plants.size(),0,0,1 );
	glTranslatef(0,-0.2,0);
	stel.bind();
	glBegin(GL_QUADS);
	glColor3f(1.0,1.0,1.0);
	glTexCoord2f( 0, 0 );
	glVertex2f( -stelsize, 2.0+stelsize*2 );
	glTexCoord2f( 1, 0 );
	glVertex2f( stelsize, 2.0+stelsize*2 );
	glTexCoord2f( 1, 1 );
	glVertex2f( stelsize, 2 );
	glTexCoord2f( 0, 1 );
	glVertex2f( -stelsize, 2 );
	glEnd();
	stel.unbind();
	glPopMatrix();
//     glBegin(GL_TRIANGLE_FAN);
//     glColor3f(0.4,0.6,0.4);
//     //    glTexCoord2f(0,1);
//     glVertex2f(0,0);
//     const double baseRadius = 2.0;
//     int segments = 60;
//     for( i  = 0 ; i<= segments; i++ )
//       {
//         double angle = i / float(segments ) * 2.0 * M_PI;
//         double noisepos[2]  = { 4.0*sin( angle ) , 4.0*cos(angle) };
//         double r = baseRadius + 0.3 * (1.0+noise2( noisepos ));
//         //        glTexCoord2f(angle / 2.0 / M_PI , 0.0 );
//         glVertex2f( r * sin(angle) , r*cos(angle) );
//       }
//     glEnd();

	//    gras.unbind();
	//    cameraFocus = Vec2d(0,0);
	glPopMatrix();
}

private:
double now;
vector<SimpelPlant*> plants;
};

Planet planet;

void display()
{
	//  printf("display\n");
	bool doneplaying =   ssystem.update();
	//  printf("donepl=%d\n", doneplaying );

	static double angle=0.0;
	double now = ftime();
	static double lasttime=0.0;
	if (capturefake)
	{
		now = lasttime + 1.0/25.0;
		if (!capture->captureEnabled)
			capture->enableCapture();
	}
	double ftimedelta = now - lasttime;
	lasttime = now;

	angle = fmod(now, 360.0 );
	//  printf("display: %f\n", angle);
	clear_and_init();

	glScalef(1.5,1.5,1.0);

	glScalef( cameraZoomFactor, cameraZoomFactor, 1.0 );
	double origfac = pow(0.50, ftimedelta );

	cameraZoomFactor = origfac * cameraZoomFactor + (1.0-origfac) * ( targetZoomFactor );

	//  glScalef( 1.0/cameraFocus[1] , 1.0/cameraFocus[1], 1.0);
	if (!CGLA::isnan( cameraFocus[0] ) )
	{
		smoothCamera = origfac * smoothCamera + (1.0-origfac ) *  ( cameraFocus );
		//      printf("\nsmoothCamera = %f,%f  cameraFocus= %f,%f\n",
		//             smoothCamera[0], smoothCamera[1], cameraFocus[0], cameraFocus[1] );

		double camlimit = -3.0;
		if (smoothCamera[1] < camlimit)
			smoothCamera[1] = camlimit; // pow(smoothCamera[1]-camlimit,2.0) + camlimit;
	}
	glTranslatef(-smoothCamera[0], -smoothCamera[1],0);
	// todo: add offset for tree

	//  glTranslatef( 0, 1.4,0);

	//  glScalef( 3.0+2.7*sin(now),3.0+2.7*sin(now),1.0);
	//glScalef( 4.0, 4.0, 1.0 );
	//  glTranslatef(0,8,0);
	glDisable( GL_DEPTH_TEST );
	glDepthMask( 0 );
	glDepthFunc( GL_LEQUAL );

	drawBackdrop( now, ftimedelta );

	glPushMatrix();
	glLoadIdentity();
	glTranslatef(-11.0, 9.0, 0.0 );

	// frame time counter
//   char buf[100];
//   sprintf(buf,"%4.1f", now );
//   glColor3f(1,1,1);
//   frend->draw(buf, Vec3d(1,1,1), Vec3d(1,0,0), 1.0);

	glPopMatrix();

	glDisable(GL_BLEND);
	glEnable(GL_BLEND);
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	//  glutWireTeapot(0.6);

	//  glDisable( GL_CULL_FACE );
	printf("delta=%f msec (%4.4f fps)  \r", 1000.0*ftimedelta, 1.0 / ftimedelta);
	const double zoomStelTime = 108.0;
	const double slingerGrowStart = 16.3;
	const double slingerGrowDuration = 30.0;
	const double transitionTime = 5.0;
	//  fast
//   const double zoomStelTime = 20.0;
//   const  double slingerGrowStart = 1.63;
//   const double slingerGrowDuration = 3.00;
//   const double transitionTime = 0.5;

	if (now< slingerGrowStart )
	{
		cameraFocus = Vec2d( 0,-10 );
	}
	if (now>slingerGrowStart && now<(slingerGrowStart + slingerGrowDuration + transitionTime) )
	{
		proot.update( now, ftimedelta );
		//
		proot.draw( Vec2d( 0,-9 ), Mat2x2d( 1,0,0,1) );
		cameraFocus = plantTop + Vec2d( 0, 0.0 );
	}

	if (now>= (slingerGrowStart + slingerGrowDuration + transitionTime) )
	{
		targetZoomFactor = 2.0;
	}

	if (now>(slingerGrowStart + slingerGrowDuration))
	{
		planet.update(now,ftimedelta);
		glPushMatrix();
		glTranslatef( 0, 30, 0 );
		planet.draw();
		glPopMatrix();
		cameraFocus = Vec2d( 0,30 );
	}

	if (now > 80 && now < 90 )
	{
		double an = now * 3.0 / 360.0 * 2.0 * M_PI;
		cameraFocus = Vec2d( 0,30) + 2.0 * Vec2d( sin( -an ), cos(-an) );
		targetZoomFactor = 4.0;
	}

	if (now > zoomStelTime - transitionTime )
	{
		cameraFocus = Vec2d( 0, 30.5 );
		targetZoomFactor = 8.0;
	}

	if (now > zoomStelTime + 10.0 )
	{
		cameraFocus = Vec2d( 0, 30.0 );
		targetZoomFactor = 32.0;
	}
	//  proot.draw( Vec2d( 3,-9 ), Mat2x2d( 1,0,0,1) );


//   glBegin(GL_LINES);
//   glColor3f(1,0,0);
//   glVertex2f( -20, cameraFocus[1] );
//   glVertex2f(  20, cameraFocus[1] );
//   glVertex2f(  cameraFocus[0], -20 );
//   glVertex2f(  cameraFocus[0], 20 );
//   glEnd();


//   glPushMatrix();
//   glLoadIdentity();
//   glBegin(GL_LINES);
//   glColor3f(0,0,0);
//   for(int i=0;i<ssystem.SPECTRUMSIZE; i++)
//     {
//       glVertex2f( -10.0+20.0*(i) / float(ssystem.SPECTRUMSIZE),  -10.0 );
//       glVertex2f( -10.0+20.0*(i) / float(ssystem.SPECTRUMSIZE),  -10.0+50.0*ssystem.spectrum[i] );
//     }
//   glEnd();
//   glPopMatrix();
	glPushMatrix();
	//  glLoadIdentity();
	toverlays->draw( now );
	glPopMatrix();

	capture->update();
	glutSwapBuffers();
}

void idle()
{
	glutPostRedisplay();
}

void keyboard( unsigned char i, int x, int y )
{
	if (i=='c')
	{
		capture->enableCapture();
	}
	if (i==27)
	{
		capture->disableCapture();
		exit(0);
	}
}


int main(int argc, char**argv)
{
	if (argc==2 && !strcmp(argv[1], "capture"))
	{
		capturefake = 1;
		printf("enabling capture support!\n");
	}
	ssystem.load("DISCODEL.XM");
	smoothCamera = Vec2d(0.0,-2.0);
	glutInit( &argc, argv );
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_MULTISAMPLE);
	glutInitWindowSize( 720, 576 );
	glutCreateWindow("view");
	capture = new Capture( 720, 576 );

	frend = new FontRenderer( "fonts/Blomster.ttf" );
	toverlays = new TextOverlayManager( );

	//  Vec3d textColor( 0.8 +0.15,0.3+0.15,0.6+0.15 );
	Vec3d textColor( 1,1,1 );
	//Vec3d textColor(0.32,0.60,0.35);
	//  textColor *= 1.3;
	//  Vec3d textColor( 1,1,1 );
	toverlays->add(  TextOverlay( 0.0, 4.0, Vec2d(0,1), 1.5, "LIMP NINJA",textColor) );
	toverlays->add(  TextOverlay( 0.0, 4.0, Vec2d(0,-0.5), 0.75, "PRESENTS",textColor));

	toverlays->add(  TextOverlay( 4.0, 8.0, Vec2d(0,1), 1.5, "A SMALL WEDTRO",textColor) );
	toverlays->add(  TextOverlay( 4.0, 8.0, Vec2d(0,-0.5), 0.75, "FOR ...",textColor));

	toverlays->add(  TextOverlay( 8.0, 12.0, Vec2d(-1.3,1), 2.5, "MLEIN",Vec3d(1,1,1), 1 ) );
	toverlays->add(  TextOverlay( 8.0, 12.0, Vec2d(-1.3,-1.25), 2.5, "&",Vec3d(1,1,1)) );
	toverlays->add(  TextOverlay( 8.0, 12.0, Vec2d(-1.3,-3.5), 2.5, "SHARKY-",Vec3d(1,1,1),-1) );
	//  toverlays->add(  TextOverlay( 8.0, 12.0, Vec2d(0,-0.5), 0.75, "FOR ..." ,Vec3d(1,1,1)));

//   toverlays->add(  TextOverlay( 0.0, 10.0, Vec2d(0,0), 1.0, "LOW1" ) );
//   toverlays->add(  TextOverlay( 10.0, 20.0, Vec2d(0,2), 1.0, "LOW2" ) );

//   toverlays->add(  TextOverlay( 30.0, 100.0, Vec2d(0,30), 0.5, "PLANET" ,Vec3d(1,0.6,0.6), 0,0 ));
//   toverlays->add(  TextOverlay( 30.0, 100.0, Vec2d(0,30.5), 0.5, "OMG" ,Vec3d(1,0.6,0.6), 0,0 ));
//   toverlays->add(  TextOverlay( 30.0, 100.0, Vec2d(0,29.5), 0.5, "LOL" ,Vec3d(1,0.6,0.6), 0,0 ));

	toverlays->add(  TextOverlay( 60.0, 100.0, Vec2d(0,30-2.7), 0.45, "WE WISH YOU A WONDERFUL TIME TOGETHER",Vec3d(1,0.8,0.2), 0,0 ));

	toverlays->add(  TextOverlay( 118.0, 200.0, Vec2d(0,30.0+0.1), 0.05, "ENJOY THE PARTY!",Vec3d(1,0.8,0.2), 0,0 ));
	toverlays->add(  TextOverlay( 118.0, 200.0, Vec2d(0,30.0-0.1), 0.05, "-THE END-",Vec3d(1,0.8,0.2), 0,0 ));

	//  glNewList(points, GL_COMPILE);
	//  glEndList();

	ssystem.play();
	glutIdleFunc( idle );
	glutDisplayFunc( display );
	glutReshapeFunc( reshape );
	glutKeyboardFunc( keyboard );

	glutMainLoop();
	return 0;
}

